import base64
import time
import re
from playwright.sync_api import sync_playwright, TimeoutError
from loguru import logger

from src.entity.account import Account, AccountType
import requests

from src.entity.database import db
from src.ui import global_signals


def playwright_login(account: Account) -> dict:
    result_container = {'headers': {}}
    response_container = {'data': {}}
    logger.info(f'Starting login for username: {account.username}')
    try:
        with sync_playwright() as playwright:
            perform_login(playwright, account, result_container, response_container)
            return result_container['headers']
    except Exception as e:
        logger.error(f'Error during login for account {account.username}: {e}', exc_info=True)
        handle_login_failure(account)
        return {}


def perform_login(playwright, account: Account, request_container: dict, response_container: dict):
    browser = playwright.chromium.launch(headless=False)
    context = browser.new_context()
    page = context.new_page()

    page.goto(account.url, timeout=120000)

    # 填充基本的元素
    fill_form_common(page, account)

    if account.type == AccountType.jy:
        handle_jy_captcha(page, account)
    else:
        handle_generic_account(page, response_container)

    # 处理 'banner' 请求并获取响应头
    handle_banner_request(page, request_container)

    # 仅当 headers 不为空时才保存到数据库中
    if request_container['headers']:
        persistence(account, request_container['headers'])
        global_signals.user_data_updated.emit()

    close_resources(page, context, browser)


def handle_generic_account(page, response_container):
    max_retries = 3
    for attempt in range(max_retries):
        if handle_captcha(page):
            with page.expect_request(lambda request: 'loginFlow' in request.url, timeout=30000):
                page.locator("div").filter(has_text=re.compile(r"^登录$")).get_by_role("button").click()

            # 等待 loginFlow 请求的响应
            with page.expect_response(lambda response: 'loginFlow' in response.url, timeout=30000) as response_info:
                response = response_info.value
                data = response.json()
                response_container['data'] = data
                if response_container['data'] and response_container['data'].get('status_code') != 6000:
                    logger.warning(f"登录发生错误, {response_container['data'].get('message', '未知错误')}")
                    close_button = page.locator("button.ant-modal-close")
                    close_button.wait_for()
                    close_button.click()
                else:
                    break
    else:
        logger.warning("登录失败，达到最大重试次数")


def handle_banner_request(page, request_container):
    with page.expect_request(lambda request: 'banner' in request.url, timeout=30000) as request_info:
        banner_request = request_info.value

    with page.expect_response(lambda response: 'banner' in response.url, timeout=30000) as response_info:
        banner_response = response_info.value
        headers = banner_request.headers
        request_container['headers'] = headers
        logger.info(f'Successfully got headers: {headers}')


def fill_form_common(page, account: Account):
    username_input = page.get_by_placeholder('用户名')
    password_input = page.get_by_placeholder('密码')
    username_input.click()
    username_input.fill(account.username)
    password_input.click()
    password_input.fill(account.password)


def handle_jy_captcha(page, account, retry_count=0):
    if retry_count < 3:
        page.locator("div").filter(has_text=re.compile(r"^登录$")).get_by_role("button").click()
        validate_code = page.locator('.geetest_box')
        validate_code.wait_for(state='visible')
        time.sleep(1)
        validate_code_buffer = validate_code.screenshot()
        img = base64.b64encode(validate_code_buffer).decode('utf-8')
        res = base64_api(img=img, typeid=33)
        drag_btn = validate_code.locator(".geetest_btn")
        drag_btn_box = drag_btn.bounding_box()
        try:
            distance = int(res) + 3
            target_x = drag_btn_box['x'] + distance
            drag_btn.hover()
            page.mouse.down()
            page.mouse.move(target_x, drag_btn_box["y"], steps=25)
            page.mouse.up()
            validate_code.wait_for(state='hidden')
            return True
        except (ValueError, TimeoutError):
            retry_count += 1
            logger.error(f"验证码处理失败，重试: {retry_count}")
            return handle_jy_captcha(page, account, retry_count)
    else:
        logger.error("验证码处理失败")


def handle_captcha(page):
    validate_code_input = page.get_by_placeholder('请输入验证码')
    validate_code_img = page.locator('#boxModle')
    validate_code_img.wait_for()
    validate_code_buffer = validate_code_img.screenshot()
    img_base64 = base64.b64encode(validate_code_buffer).decode('utf-8')
    res = base64_api(img=img_base64, typeid=1003)
    validate_code_input.click()
    validate_code_input.fill(res)
    return True


def close_resources(page, context, browser):
    try:
        if page and not page.is_closed():
            page.close()
        if context:
            context.close()
        if browser:
            browser.close()
    except Exception as e:
        logger.error(f'Error closing browser resources: {e}', exc_info=True)


def handle_login_failure(account: Account):
    pass


def base64_api(uname='luffy230505', pwd='qwer12345', img='', typeid=20):
    logger.info('Calling third part interfaces')
    data = {
        "username": uname,
        "password": pwd,
        "typeid": typeid,
        "image": img,
        'softid': '8d13df0efe074035b54ee9c2bef85106'
    }
    response = requests.post("http://api.ttshitu.com/predict", json=data)
    result = response.json()
    if result['success']:
        return result["data"]["result"]
    else:
        return result["message"]


def login(account: Account):
    headers = playwright_login(account)
    if headers:
        account.headers = headers
        return account
    else:
        return account


def persistence(account: Account, headers: dict):
    logger.info(f'Persisting headers for account {account.name}')
    with db.Session() as session:
        db_account = session.query(Account).filter(Account.username == account.username,
                                                   Account.password == account.password).one()
        db_account.headers = headers
        session.commit()
        logger.info(f'Headers persisted for account {account.name}')
