import time
from enum import Enum

from loguru import logger
from PyQt6.QtCore import QDate, QDateTime, Qt, QThread, QTimer, pyqtSignal, QTime
from PyQt6.QtGui import QAction, QIcon, QColor
from PyQt6.QtWidgets import (
    QCheckBox, QDateEdit, QHBoxLayout, QHeaderView, QMainWindow, QMenu,
    QMessageBox, QPushButton, QSystemTrayIcon, QTableWidget, QTableWidgetItem,
    QTabWidget, QTextEdit, QVBoxLayout, QWidget, QSizePolicy, QApplication
)

from src.core.message_client import send_message
from src.ui.config import ConfigManager
from src.core.util import resource_path, convert_data
from src.entity.user import get_user_by_username_and_password
from src.entity.member import get_today_new_member_list
from src.entity.pay_record import get_latest_deposit_user
from src.ui import global_signals
from src.ui.data_query import ButtonTask, ReportTask
from src.ui.thread_pool_manager import pyqt_thread_pool
from src.ui.title_bar import CustomTitleBar


class DataLoaderThread(QThread):
    tab_ready = pyqtSignal(str, list)

    def __init__(self, user):
        super().__init__()
        self.user = user

    def run(self):
        data = ReportTask.get_banner_info_by_user(self.user)
        for row in data:
            time_str = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
            self.tab_ready.emit(row.agentCode, [time_str, row.registerMembers, row.firstDepositNum, row.netWinLose,
                                                row.effectiveNew, row.activeMembers])


class Application(QMainWindow):
    def __init__(self):
        super().__init__()
        self.config_manager = ConfigManager()
        self.username = None
        self.password = None
        self.user = None
        self.tables = {}
        self.toaster_notify_enabled = True
        self.telegram_notify_enabled = True
        self.is_dragging = False
        self.drag_position = None
        self.is_resizing = False
        self.resize_direction = None

        self.init_ui()
        self.load_config()
        self.setup_timers()
        self.connect_signals()
        self.load_data()

    def init_ui(self):
        self.set_window_properties()
        self.create_central_widget()
        self.setup_layouts()
        self.apply_stylesheet()

    def set_window_properties(self):
        self.resize(800, 600)
        self.setWindowIcon(QIcon("icons:icon.png"))
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint)

    def create_central_widget(self):
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        self.main_layout = QVBoxLayout(self.central_widget)

    def setup_layouts(self):
        self.top_panel = QHBoxLayout()
        self.setup_top_panel()
        self.main_layout.addLayout(self.top_panel)

        self.notebook = QTabWidget()
        self.main_layout.addWidget(self.notebook)

        self.bottom_panel = QVBoxLayout()
        self.txt = QTextEdit()
        self.bottom_panel.addWidget(self.txt)
        self.main_layout.addLayout(self.bottom_panel)

    def setup_top_panel(self):
        self.add_buttons_to_top_panel()
        self.add_date_picker_to_top_panel()
        self.add_checkboxes_to_top_panel()
        self.setup_report_button()

    def add_buttons_to_top_panel(self):
        buttons_info = [
            ("报数", "Primary"),
            ("存款失败用户", "Danger"),
            ("负盈利", "Success"),
            ("薪资", "Light"),
        ]
        for name, style in buttons_info:
            button = QPushButton(name)
            button.setObjectName(style)
            button.clicked.connect(lambda _, n=name: self.query_data(n))
            self.top_panel.addWidget(button)

    def add_date_picker_to_top_panel(self):
        self.dateEdit = QDateEdit()
        self.dateEdit.setCalendarPopup(True)
        self.dateEdit.setDate(QDate.currentDate())
        today = QDate.currentDate()
        first_day_last_month = QDate(today.year(), today.month(), 1).addMonths(-1)
        self.dateEdit.setMinimumDate(first_day_last_month)
        self.dateEdit.setMaximumDate(today)
        self.top_panel.addWidget(self.dateEdit)

    def add_checkboxes_to_top_panel(self):
        checkbox_layout = QVBoxLayout()
        self.system_notification_checkbox = QCheckBox("系统通知")
        self.telegram_notification_checkbox = QCheckBox("飞机通知")
        self.system_notification_checkbox.setChecked(True)
        self.telegram_notification_checkbox.setChecked(True)
        self.system_notification_checkbox.stateChanged.connect(self.toggle_system_notification)
        self.telegram_notification_checkbox.stateChanged.connect(self.toggle_telegram_notification)
        checkbox_layout.addWidget(self.system_notification_checkbox)
        checkbox_layout.addWidget(self.telegram_notification_checkbox)
        self.top_panel.addLayout(checkbox_layout)

    def setup_report_button(self):
        self.report_button = QPushButton("停止喜报")
        self.report_button.setCheckable(True)
        self.report_button.setChecked(True)
        self.report_button.setObjectName("Warning")
        self.report_button.clicked.connect(self.on_report_clicked)
        self.top_panel.addWidget(self.report_button)

    def init_tray_icon(self):
        self.tray_icon = SystemTrayIcon(QIcon("icons:icon.png"), self)
        self.tray_icon.setToolTip(self.windowTitle())
        self.tray_icon.show()

    def load_config(self):
        try:
            self.username = self.config_manager.get('Credentials', 'username')
            self.password = self.config_manager.get('Credentials', 'password')
            self.user = get_user_by_username_and_password(self.username, self.password)
            self.setWindowTitle(f'{self.username}的小工具')
            self.customTitleBar = CustomTitleBar(self)
            self.setMenuWidget(self.customTitleBar)
            self.init_tray_icon()  # 在设置标题后初始化托盘图标
        except FileNotFoundError:
            QMessageBox.warning(self, "警告", "配置文件信息加载失败！请检查文件是否存在")

    def setup_timers(self):
        self.report_timer = QTimer(self)
        self.report_timer.timeout.connect(self.update_reports)
        self.report_timer.start(60000)
        self.date_update_timer = QTimer(self)
        self.date_update_timer.timeout.connect(self.update_date_edit)
        self.start_date_update_timer()

    def start_date_update_timer(self):
        now = QDateTime.currentDateTime()
        next_midnight = QDateTime(now.date().addDays(1), QTime(0, 0))
        interval = now.msecsTo(next_midnight)
        self.date_update_timer.start(interval if interval > 0 else 86400000)

    def update_date_edit(self):
        self.dateEdit.setDate(QDate.currentDate())
        self.update_date_range()
        self.start_date_update_timer()

    def update_date_range(self):
        today = QDate.currentDate()
        first_day_last_month = QDate(today.year(), today.month(), 1).addMonths(-1)
        self.dateEdit.setMinimumDate(first_day_last_month)
        self.dateEdit.setMaximumDate(today)

    def connect_signals(self):
        self.tray_icon.activated.connect(self.tray_icon_clicked)
        global_signals.user_data_updated.connect(self.refresh_user_data)

    def load_data(self):
        self.data_loader_thread = DataLoaderThread(self.user)
        self.data_loader_thread.tab_ready.connect(self.add_tab)
        self.data_loader_thread.start()

    def add_tab(self, account_username, data):
        if account_username not in self.tables:
            tab = QWidget()
            tab_layout = QVBoxLayout(tab)
            table = self.create_table_for_tab()
            tab_layout.addWidget(table)
            self.tables[account_username] = table
            account_name = self.get_account_name(account_username)
            self.notebook.addTab(tab, account_name)
        self.update_table(account_username, data)

    def get_account_name(self, account_username):
        for account in self.user.accounts:
            if account.username == account_username:
                return account.name
        return account_username

    def create_table_for_tab(self):
        table = QTableWidget()
        column_headers = ColumnHeaders.list()
        table.setColumnCount(len(column_headers))
        table.setHorizontalHeaderLabels(column_headers)
        table.setEditTriggers(QTableWidget.EditTrigger.NoEditTriggers)
        header = table.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)
        table.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding))
        return table

    def update_table(self, account_username, data):
        try:
            table = self.tables[account_username]
            self.ensure_table_row_limit(table)
            self.insert_data_in_table(table, data, account_username)
        except Exception as e:
            logger.error(f"Error updating table for {account_username}: {e}")

    def ensure_table_row_limit(self, table, row_limit=20):
        while table.rowCount() >= row_limit:
            table.removeRow(0)

    def insert_data_in_table(self, table, data, account_username):
        row_count = table.rowCount()
        table.insertRow(row_count)
        notifications = []
        for col, cell_data in enumerate(data):
            self.set_table_item(table, row_count, col, cell_data)
            self.check_data_change_for_notifications(table, row_count, col, cell_data, account_username, notifications)
        self.send_all_notifications(notifications)

    def set_table_item(self, table, row, col, cell_data):
        cell = QTableWidgetItem(str(cell_data))
        cell.setTextAlignment(Qt.AlignmentFlag.AlignCenter)
        table.setItem(row, col, cell)

    def check_data_change_for_notifications(self, table, row, col, new_data, account_username, notifications):
        if row > 0 and col != 0:
            old_data_str = table.item(row - 1, col).text()
            old_data = convert_data(old_data_str)
            new_data = convert_data(new_data)
            if old_data != new_data:
                count_change = new_data - old_data
                self.update_cell_color(table, row, col, count_change)
                self.generate_notifications(account_username, col, new_data, count_change, notifications)

    def update_cell_color(self, table, row, col, count_change):
        cell = table.item(row, col)
        if cell:
            if count_change > 0:
                cell.setForeground(QColor(Qt.GlobalColor.green))
            elif count_change < 0:
                cell.setForeground(QColor(Qt.GlobalColor.red))

    def generate_notifications(self, account_username, col, cell_data, count_change, notifications):
        account = self.get_account(account_username)
        if count_change > 0:
            if col == 1:
                reg_results = ','.join(
                    [f'`{member.name}`' for member in get_today_new_member_list(account, count_change)])
                notifications.append(
                    ('👏', account.name, '注册', count_change, f'用户: {reg_results}', str(cell_data)))
            elif col == 2:
                deposit_results = '\n'.join(
                    [f"用户: `{member.name}`, 首存金额: *{member.deposit}*" for member in
                     get_latest_deposit_user(account, count_change)])
                notifications.append(
                    ('🎉', account.name, '首存', count_change, deposit_results, str(cell_data)))

    def get_account(self, account_username):
        for account in self.user.accounts:
            if account.username == account_username:
                return account
        return None

    def send_all_notifications(self, notifications):
        for notification in notifications:
            self.send_notification(*notification)

    def on_report_clicked(self):
        if self.report_button.isChecked():
            self.report_timer.start(60000)
            self.report_button.setText("停止喜报")
            self.report_button.setObjectName("Warning")
        else:
            self.report_timer.stop()
            self.report_button.setText("启动喜报")
            self.report_button.setObjectName("Success")
        self.apply_stylesheet()

    def tray_icon_clicked(self, reason):
        if reason == QSystemTrayIcon.ActivationReason.Trigger:
            if self.isMinimized() or not self.isVisible():
                self.showNormal()
                self.activateWindow()
            else:
                self.hide()

    def refresh_user_data(self):
        self.user = get_user_by_username_and_password(self.username, self.password)
        self.setWindowTitle(f'{self.username}的小工具')

    def apply_stylesheet(self):
        style_sheet = self.load_stylesheet(resource_path('ui/style.qss'))
        self.setStyleSheet(style_sheet)

    @staticmethod
    def load_stylesheet(file_path):
        try:
            with open(file_path, "r", encoding='utf-8') as file:
                return file.read()
        except FileNotFoundError:
            print(f"无法找到样式文件: {file_path}")
            return ""

    def toggle_system_notification(self, state):
        self.toaster_notify_enabled = state == Qt.CheckState.Checked

    def toggle_telegram_notification(self, state):
        self.telegram_notify_enabled = state == Qt.CheckState.Checked

    def update_reports(self):
        for account in self.user.accounts:
            report_task = ReportTask(account)
            report_task.signals.table_updated.connect(self.update_table)
            pyqt_thread_pool.start(report_task)

    def query_data(self, btn_name):
        selected_date = self.dateEdit.date()
        selected_date_str = selected_date.toString("yyyy-MM-dd")
        self.txt.append(f"正在查询{selected_date_str}的{btn_name}，请等待...\n")
        task = ButtonTask(btn_name, selected_date_str, self.user)
        task.signals.query_completed.connect(self.display_query_result)
        pyqt_thread_pool.start(task)

    def display_query_result(self, result, auto_clipboard, need_notify):
        if auto_clipboard:
            copy_to_clipboard(result)
        if need_notify and '无' not in result:
            self.tray_icon.showMessage("", "自动复制成功", QSystemTrayIcon.MessageIcon.Information, 500)
        self.txt.append(result)

    def send_notification(self, emoji, account_name, title, count, results, total):
        msg = f'{emoji} {account_name} {title}:{count} {results} 总数:*{total}*'
        if self.toaster_notify_enabled:
            self.tray_icon.showMessage(f"{title}通知", msg, QSystemTrayIcon.MessageIcon.Information, 2000)
        if self.telegram_notify_enabled:
            send_message(self.user.bot_token, self.user.group_id, msg)

    def exit_application(self):
        self.tray_icon.hide()
        QApplication.quit()

    def closeEvent(self, event):
        event.ignore()
        self.hide()
        self.tray_icon.showMessage(
            "最小化到托盘",
            "应用程序已最小化到托盘。要退出，请使用托盘菜单。",
            QSystemTrayIcon.MessageIcon.Information,
            2000
        )

    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.drag_position = event.globalPosition().toPoint()
            self.resize_direction = self.get_resize_direction(event.pos())
            if self.resize_direction:
                self.is_resizing = True
            elif self.is_draggable_area(event.pos()):
                self.is_dragging = True
            else:
                return
        super().mousePressEvent(event)

    def is_draggable_area(self, pos):
        title_bar_pos = self.customTitleBar.mapFromParent(pos)
        return self.customTitleBar.rect().contains(title_bar_pos)

    def reset_cursor_style(self):
        if not (self.is_dragging or self.is_resizing):
            self.setCursor(Qt.CursorShape.ArrowCursor)

    def mouseMoveEvent(self, event):
        direction = self.get_resize_direction(event.pos())
        if direction == "left" or direction == "right":
            self.setCursor(Qt.CursorShape.SizeHorCursor)
        elif direction == "top" or direction == "bottom":
            self.setCursor(Qt.CursorShape.SizeVerCursor)
        elif direction in ["top-left", "bottom-right"]:
            self.setCursor(Qt.CursorShape.SizeFDiagCursor)
        elif direction in ["top-right", "bottom-left"]:
            self.setCursor(Qt.CursorShape.SizeBDiagCursor)
        else:
            self.reset_cursor_style()

        if self.is_dragging:
            self.move(self.pos() + (event.globalPosition().toPoint() - self.drag_position))
            self.drag_position = event.globalPosition().toPoint()

        elif self.is_resizing:
            self.resize_window(event.globalPosition().toPoint())

        super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        self.is_dragging = False
        self.is_resizing = False
        super().mouseReleaseEvent(event)

    def get_resize_direction(self, pos):
        border_width = 10
        rect = self.rect()
        left, right, top, bottom = rect.left(), rect.right(), rect.top(), rect.bottom()

        if pos.x() < left + border_width and pos.y() < top + border_width:
            return "top-left"
        if pos.x() > right - border_width and pos.y() < top + border_width:
            return "top-right"
        if pos.x() < left + border_width and pos.y() > bottom - border_width:
            return "bottom-left"
        if pos.x() > right - border_width and pos.y() > bottom - border_width:
            return "bottom-right"
        if pos.x() < left + border_width:
            return "left"
        if pos.x() > right - border_width:
            return "right"
        if pos.y() < top + border_width:
            return "top"
        if pos.y() > bottom + border_width:
            return "bottom"
        return None

    def resize_window(self, current_pos):
        if not self.resize_direction:
            return

        delta = current_pos - self.drag_position
        rect = self.geometry()

        if "left" in self.resize_direction:
            rect.setLeft(rect.left() + delta.x())
        if "right" in self.resize_direction:
            rect.setRight(rect.right() + delta.x())
        if "top" in self.resize_direction:
            rect.setTop(rect.top() + delta.y())
        if "bottom" in self.resize_direction:
            rect.setBottom(rect.bottom() + delta.y())

        self.setGeometry(rect)
        self.drag_position = current_pos


class SystemTrayIcon(QSystemTrayIcon):
    def __init__(self, icon, parent=None):
        super().__init__(icon, parent)
        self.menu = QMenu(parent)
        self.setContextMenu(self.menu)
        self.create_actions(parent)

    def create_actions(self, parent):
        exit_action = QAction("Exit", parent)
        exit_action.triggered.connect(parent.exit_application)
        self.menu.addAction(exit_action)


class ColumnHeaders(Enum):
    TIME = "时间"
    REGISTER = "注册"
    FIRST_DEPOSIT = "首存"
    NET_PROFIT = "负盈利"
    EFFECTIVE = "有效"
    ACTIVE = "活跃"

    @classmethod
    def list(cls):
        return [cls.TIME.value, cls.REGISTER.value, cls.FIRST_DEPOSIT.value,
                cls.NET_PROFIT.value, cls.EFFECTIVE.value, cls.ACTIVE.value]


def copy_to_clipboard(text):
    clipboard = QApplication.clipboard()
    clipboard.setText(text)

